@@ -1,7 +1,7 @@
 
-/*  $Header: /cvs/gnome/gcalctool/gcalctool/functions.c,v 1.58 2004/11/11 19:22:41 richb Exp $
+/*  $Header: /cvs/gnome/gcalctool/gcalctool/functions.c,v 1.62.2.1 2005/03/14 19:08:33 richb Exp $
  *
- *  Copyright (c) 1987-2004 Sun Microsystems, Inc. All Rights Reserved.
+ *  Copyright (c) 1987-2005 Sun Microsystems, Inc. All Rights Reserved.
  *           
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -32,10 +32,15 @@
 #include "ce_parser.h"
 #include "lr_parser.h"
 
+static int do_trigfunc(int s[MP_SIZE], int t[MP_SIZE]);
+static int key_equal(struct button *, struct button);
+
 static void do_accuracy();
 static void do_constant();
 static void do_exchange();
 static void do_function();
+static void do_immedfunc(int s[MP_SIZE], int t[MP_SIZE]);
+static void do_portionfunc(int num[MP_SIZE]);
 static void do_shift();
 
 
@@ -303,6 +308,76 @@ str_replace(char **str, char *from, char *to)
     }
 }
 
+void
+trig_filter(char **func) 
+{
+    enum mode {
+        normal = 0,
+        inv = 1,
+        hyp = 2,
+        invhyp = 3,
+    } mode;
+  
+    int inverse;
+    int hyperbolic;
+
+    assert(func);
+    inverse = (v->inverse) ? inv : 0;
+    hyperbolic = (v->hyperbolic) ? hyp : 0;
+
+    mode = (inverse | hyperbolic);
+
+    if (!strcmp(*func, "Sin")) {
+        switch (mode) {
+            case normal:
+                break;
+            case inv:
+                str_replace(func, "Sin", "Asin");
+                break;
+            case hyp:
+                str_replace(func, "Sin", "Sinh");
+                break;
+            case invhyp:
+                str_replace(func, "Sin", "Asinh");
+                break;
+            default:
+                assert(0);
+        }
+    } else if (!strcmp(*func, "Cos")) {
+        switch (mode) {
+            case normal:
+                break;
+            case inv:
+                str_replace(func, "Cos", "Acos");
+                break;
+            case hyp:
+                str_replace(func, "Cos", "Cosh");
+                break;
+            case invhyp:
+                str_replace(func, "Cos", "Acosh");
+                break;
+            default:
+                assert(0);
+        }
+    } else if (!strcmp(*func, "Tan")) {
+        switch (mode) {
+            case normal:
+                break;
+            case inv:
+                str_replace(func, "Tan", "Atan");
+                break;
+            case hyp:
+                str_replace(func, "Tan", "Tanh");
+                break;
+            case invhyp:
+                str_replace(func, "Tan", "Atanh");
+                break;
+            default:
+                assert(0);
+        }
+    }
+}
+
 
 void
 do_expression()
@@ -312,6 +387,8 @@ do_expression()
     update_statusbar("", "");
 
     btext = (v->current->symname) ? v->current->symname : v->current->str;
+    btext = gc_strdup(btext);
+    trig_filter(&btext);
   
     if (v->e.calc_complete) {
         v->e.calc_complete = 0;
@@ -323,8 +400,7 @@ do_expression()
             assert(!v->expression);
             v->expression = v->e.expbak;
             v->e.expbak = NULL;
-	    refresh_display();
-	    return;
+	    goto out;
         }
 
         if (v->current->flags & 
@@ -335,8 +411,7 @@ do_expression()
 
             snprintf(buf, 128, "%s(Ans)", btext);
             exp_replace(buf);
-	    refresh_display();
-	    return;
+	    goto out;
         } else if (v->current->flags & (number | func)) {
             exp_del(); 
         }
@@ -357,8 +432,7 @@ do_expression()
         set_error_state(FALSE);
         MPstr_to_num("0", DEC, v->e.ans);
         MPstr_to_num("0", DEC, v->e.ansbak);
-	refresh_display();
-	return;
+	goto out;
     } else if (v->current->flags & regrcl) {
         int i = char_val(v->current->value[0]);
         char reg[3];
@@ -366,30 +440,25 @@ do_expression()
 
         snprintf(reg, 3, "R%c", n);
         exp_append(reg);
-	refresh_display();
-	return;
+	goto out;
     } else if (v->current->flags & con) {
         int *MPval = v->MPcon_vals[char_val(v->current->value[0])];
         exp_append(make_number(MPval, v->base, TRUE, FALSE));
-	refresh_display();
-	return;
+	goto out;
     } else if (v->current->flags & bsp) {
         if (exp_has_postfix(v->expression, "Ans")) { 
             char *ans = make_number(v->e.ans, v->base, TRUE, FALSE);   
 
             str_replace(&v->expression, "Ans", ans);
         } 
-        exp_del_char(&v->expression, 1); 
-	refresh_display();
-	return;
+        exp_del_char(&v->expression, 1);
+	goto out;
     } else if (v->current->flags & neg) {
         exp_negate();
-	refresh_display();
-	return;
+	goto out;
     } else if (v->current->flags & inv) {
         exp_inv();
-	refresh_display();
-	return;
+	goto out;
     }
 
     if (v->current->flags & enter) {
@@ -398,21 +467,20 @@ do_expression()
             int ret = ce_parse(v->expression, MPval);
 
             if (!ret) {
+	        // FIXME: duplicated code in do_exchange().
 	        mpstr(v->e.ans, v->e.ansbak);
 	        mpstr(MPval, v->e.ans);
 	        v->e.expbak = gc_strdup(v->expression);
 	        exp_replace("Ans");
 	        v->e.calc_complete = 1;
-		refresh_display();
-		return;
+		goto out;
             } else {
 	        update_statusbar(_("Malformed expression"), 
                                  "gtk-dialog-error");
 	        return;
             }
         } else {
-	    refresh_display();
-	    return;
+	  goto out;
         }
     }
 
@@ -422,7 +490,11 @@ do_expression()
         exp_append("(");
     }
 
+ out:
+    free(btext);
+    btext = NULL;
     refresh_display();
+    return;
 }
 
 
@@ -706,12 +778,45 @@ do_delete()     /* Remove the last numeric character typed. */
 static void
 do_exchange()         /* Exchange display with memory register. */
 {
-    int MPtemp[MP_SIZE];
+  int MPtemp[MP_SIZE];
+  int MPexpr[MP_SIZE];
+
+    switch (v->syntax) {
+        case npa:
+	  {
+	    mpstr(v->MPdisp_val, MPtemp);
+	    mpstr(v->MPmvals[char_val(v->current->value[0])], v->MPdisp_val);
+	    mpstr(MPtemp, v->MPmvals[char_val(v->current->value[0])]);
+	    make_registers();
+	  }
+	  break;
+
+        case exprs:
+	  {
+	    int ret = usable_num(MPexpr);
+	    int n = char_val(v->current->value[0]);
+	    if (ret) {
+	      update_statusbar(_("No sane value to store"), 
+			       "gtk-dialog-error");
+	    } else {
+	      mpstr(v->MPmvals[n], MPtemp);
+	      mpstr(MPexpr, v->MPmvals[n]);
+	      mpstr(MPtemp, v->e.ans);	      
+	      // TODO: duplicated code in do_expression
+	      mpstr(v->e.ans, v->e.ansbak);
+	      v->e.expbak = gc_strdup(v->expression);
+	      exp_replace("Ans");
+	      v->e.calc_complete = 1;
+              refresh_display();
+	      make_registers();
+	    }
+	  }
+	  break;
+
+        default:
+            assert(0);
+    }
 
-    mpstr(v->MPdisp_val, MPtemp);
-    mpstr(v->MPmvals[char_val(v->current->value[0])], v->MPdisp_val);
-    mpstr(MPtemp, v->MPmvals[char_val(v->current->value[0])]);
-    make_registers();
 }
 
 
