@@ -1,5 +1,5 @@
 
-/*  $Header: /cvs/gnome/gcalctool/gcalctool/gtk.c,v 1.155 2005/02/03 17:57:22 richb Exp $
+/*  $Header: /cvs/gnome/gcalctool/gcalctool/gtk.c,v 1.159 2005/02/15 22:20:23 richb Exp $
  *
  *  Copyright (c) 1987-2005 Sun Microsystems, Inc. All Rights Reserved.
  *           
@@ -511,7 +511,8 @@ about_cb(GtkAction *action, gpointer date)
         					"authors", authors,
         					"documenters", documenters,
         					"translator-credits", strcmp(translator_credits, "translator_credits") != 0 ? translator_credits : NULL,
-        					"logo", X->icon);
+        					"logo", X->icon,
+                                                NULL);
         					
 }
 
@@ -1977,8 +1978,10 @@ make_but_panel(GtkWidget *vbox, GtkWidget **Gtk_buttons,
                 Gtk_buttons[n] = make_menu_button(label, j*cols + i);
             }
             set_accessible_name(Gtk_buttons[n], buttons[n]);
-            g_signal_connect(G_OBJECT(Gtk_buttons[n]), "clicked",
-                            G_CALLBACK(button_proc), (gpointer) (j*cols + i));
+            if (buttons[n].mtype == M_NONE) {
+                g_signal_connect(G_OBJECT(Gtk_buttons[n]), "clicked",
+                             G_CALLBACK(button_proc), (gpointer) (j*cols + i));
+            }
             SPRINTF(name, "%s_button%1d", tag, n);
             gtk_widget_set_name(Gtk_buttons[n], name);
             if (buttons[n].hstr != NULL) {
@@ -2200,11 +2203,11 @@ mb_proc(GtkAction *action)
         do_memory();
     } else if (EQUAL(name, "Contents")) {
         help_cb();
-    } else if (EQUAL(name, "LSPlaces")) {
+    } else if (!strncmp(name, "LSPlaces", 8)) {
         SSCANF(name,"LSPlaces%d", &choice);
         choice += (choice < 10) ? '0' : 'A' - 10;
         handle_menu_selection(X->mrec[(int) M_LSHF], choice);
-    } else if (EQUAL(name, "RSPlaces")) {
+    } else if (!strncmp(name, "RSPlaces", 8)) {
         SSCANF(name,"RSPlaces%d", &choice);
         choice += (choice < 10) ? '0' : 'A' - 10;
         handle_menu_selection(X->mrec[(int) M_RSHF], choice);
